<!DOCTYPE html>
<html lang="en" xmlns="http://www.w3.org/1999/xhtml" xmlns:v="urn:schemas-microsoft-com:vml" xmlns:o="urn:schemas-microsoft-com:office:office">
<head>
    <meta charset="utf-8"> <!-- utf-8 works for most cases -->
</head>

<body>
    <center>
        <p style="font-size: 8pt;">
            ${subject_info}
        </p>

    </center>
    <hr>
    <span>${date} - ${hour}</span>
    <span class="right">Faktura sinp.: ${serial_code}/${num}</span>
    <hr>
    <table style="font-size: 6pt;">
        <tr>
            <th>Kop.</th>
            <th>Deskribapena</th>
            <th>Unitatea</th>
            <th>Guztira</th>
        </tr>
        <tr class="invoice_lines">
        </tr>
    </table>
    <hr>
    <div style="font-size: 10pt;">
    <span><strong>Guztira BEZ barne</strong></span><span class="right"><strong>${invoice_total} €</strong></span>
    </div>
    </br>
    </br>
    <p>
        <span>BEZ<span>

    </p>
    <hr>
    <center style="font-size: 9pt;">
        <strong>ESKERRIK ASKO</strong>
    </center>
    <hr>
    <center>
        <p><strong style="font-size: 7pt;">${tbai_id}</strong></p>
        <img src="data:image/jpeg;base64,${qrcode}"/>
    </center>
</body>
</html>
